/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * EE to test channels
 */

#undef CHAN_ETH
#undef USEIFCHAN	/* use "if" instead of "eth" */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_mempool.h>
#include <nodeos/an_misc.h>
#include <nodeos/an_pbuf.h>

static void ee_init(void *arg);
an_flowinit_func_t InitEE = { ee_init };

static void setup_channels(void);
static void ee_packet1(an_pbuf_t *pbuf, void *arg);
static void ee_packet100(an_pbuf_t *pbuf, void *arg);
static an_chan_t *chan1, chan1_data, *chan100, chan100_data;
static an_pbuf_t *buf1, buf1_data, *buf100, buf100_data;
#ifdef CHAN_ETH
static void ee_eth(void);
static void ee_realpacket(an_pbuf_t *pbuf, void *arg);
static an_ifinfo_t info[8];
static struct ethstuff {
	an_chan_t eth_data;
	an_chan_t *eth;
	an_pbuf_t ethbuf_data;
	an_ifinfo_t *info;
} *eth;
#endif

static struct {
	an_chanaddr_t addr;
	an_chanproto_t proto;
	an_cpuspec_t cpu;
	an_pbuf_t *bufs[1];
} attr;
static an_flow_t *flow;

int
ee_main(int argc, const char **argv)
{
	flow = an_flow_current();
	printf("%p/%p: Channel test EE started\n",
	       flow, an_thread_current());

	/*
	 * Setup channels
	 */
	setup_channels();

	/*
	 * No longer need this thread
	 */
	return 0;
}

static void
setup_channels(void)
{
	memset(&attr, 0, sizeof attr);

#ifdef CHAN_ETH
	ee_eth();
#endif

	/*
	 * Create packet buffers
	 */
	buf1 = an_pbuf_create(&buf1_data, AN_PBUF_CLOCK, 0, 0);
	assert(buf1 != 0);
	buf100 = an_pbuf_create(&buf100_data, AN_PBUF_CLOCK, 0, 0);
	assert(buf100 != 0);

	strcpy(attr.proto.data, "clock");
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.clock.freq = 100;
	attr.cpu.max = 1;
	attr.bufs[0] = buf100;
	chan100 = an_inchan_create(&chan100_data, flow, 0,
				   &attr.proto, &attr.addr, attr.bufs, 1,
				   &attr.cpu, ee_packet100, &chan100_data);
	assert(chan100 != 0);

	attr.addr.clock.freq = 1;
	attr.bufs[0] = buf1;
	chan1 = an_inchan_create(&chan1_data, flow, 0,
				 &attr.proto, &attr.addr, attr.bufs, 1,
				 &attr.cpu, ee_packet1, &chan1_data);
	assert(chan1 != 0);
}

static int inpackets;

static void
ee_packet1(an_pbuf_t *pbuf, void *arg)
{
	assert(arg == chan1);

	inpackets++;
	an_inchan_addpbuf(chan1, pbuf);
}

static void
ee_packet100(an_pbuf_t *pbuf, void *arg)
{
	static int count;
	int size = an_pbuf_size(pbuf);
	unsigned int *ts = an_pbuf_data(pbuf);
	an_timespec_t tspec;
	
	assert(arg == chan100);
	assert(size == sizeof *ts);

	an_gettime(&tspec);
	printf("%p/%p: packet %p@%u: got %d packets at %u seconds\n",
	       an_flow_current(), an_thread_current(), pbuf, *ts, inpackets,
	       (unsigned int)tspec.tv_sec);
	an_inchan_addpbuf(chan100, pbuf);
	if (++count == 20) {
		printf("killing timer channel %p\n", chan1);
		if (chan1)
			an_chan_destroy(chan1);
/* if we kill our channel, the flow won't terminate */
#if 0
		printf("killing our channel %p\n", chan100);
		if (chan100)
			an_chan_destroy(chan100);
#else
		an_flow_destroy(an_flow_current());
#endif
		printf("WE ARE DEAD!\n");
	}
}

#ifdef CHAN_ETH
static void
ee_eth(void)
{
	int nif = 8;
	int err, i;

	/*
	 * Find out interfaces.
	 */
	err = an_ifinfo_get(info, &nif);
	assert(err == 0);

	for (i = 0; i < nif; i++) {
		assert(info[i].type == ANIF_ETH);

#if 1
		printf("if%d: %s: %02x:%02x:%02x:%02x:%02x:%02x\n",
		       info[i].id, info[i].pname,
		       info[i].addr.eth.daddr[0], info[i].addr.eth.daddr[1],
		       info[i].addr.eth.daddr[2], info[i].addr.eth.daddr[3],
		       info[i].addr.eth.daddr[4], info[i].addr.eth.daddr[5]);
#endif
#ifdef USEIFCHAN
		sprintf(attr.proto.data, "if%d", info[i].id);
#if 1
		/* get broadcasts as well as directed packets */
		memset(&attr.addr, 0, sizeof attr.addr);
#else
		/* directed packets only */
		attr.addr = info[i].addr;
#endif
#else
		strncpy(attr.proto.data, info[i].pname, sizeof info[i].pname);
#endif
		eth = an_mempool_alloc(an_mempool_current(), sizeof *eth);
		assert(eth != 0);
		eth->info = &info[i];

		attr.proto.len = strlen(attr.proto.data);
		attr.cpu.max = 1;
		attr.bufs[0] = an_pbuf_init(&eth->ethbuf_data,
					    AN_PBUF_ETH, 0, 0);
		eth->eth = an_inchan_create(&eth->eth_data, flow, 0,
				       &attr.proto, &attr.addr, attr.bufs, 1,
				       &attr.cpu, ee_realpacket, eth);
		assert(eth != 0);
	}
}

static void
ee_realpacket(an_pbuf_t *pbuf, void *arg)
{
	struct ethstuff *eth = arg;

#ifdef USEIFCHAN
	unsigned int *ip = an_pbuf_data(pbuf);

	assert(eth->info != 0);
	printf("%p/%p: if%d: %8.8x %8.8x\n",
	       an_flow_current(), an_thread_current(), eth->info->id,
	       ip[0], ip[1]);
#else
	int size = an_pbuf_size(pbuf);
	struct {
		unsigned char daddr[6];
		unsigned char saddr[6];
		short type;
	} *hdr = an_pbuf_data(pbuf);

	assert(eth->info != 0);
	assert(size >= sizeof *hdr);

	printf("%p/%p: %s: "
	       "%02x:%02x:%02x:%02x:%02x:%02x -> %02x:%02x:%02x:%02x:%02x:%02x"
	       ": %x\n",
	       an_flow_current(), an_thread_current(), eth->info->pname,
	       hdr->saddr[0], hdr->saddr[1], hdr->saddr[2],
	       hdr->saddr[3], hdr->saddr[4], hdr->saddr[5],
	       hdr->daddr[0], hdr->daddr[1], hdr->daddr[2],
	       hdr->daddr[3], hdr->daddr[4], hdr->daddr[5], ntohs(hdr->type));
#endif

	an_inchan_addpbuf(eth->eth, pbuf);
}
#endif

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "ChantestEE" }; 

	status = ee_main(argc, argv);
}

