/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Reply to UDP ping packets (shamelessly stolen from OSKit pingreply).
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_mutex.h>
#include <nodeos/an_pbuf.h>
#include <nodeos/an_misc.h>

#include <nodeos/impl/an_gprof.h>	/* XXX */

#define RUNFOR		60	/* run for this many seconds */
#define MAXTHREADS	16	/* max threads handling packets at once */
#define MAXBUFS		32	/* max packet buffers for those threads */

static unsigned char myip[4] = { 155, 99, 214, 158 };


/*
 * For some reason the headers define ETHER_MAX_LEN as
 * (ETH_FRAME_LEN + ETH_CRC_LEN), but then don't define ETH_CRC_LEN.
 */
#ifdef __linux__
#ifndef ETH_CRC_LEN
#define ETH_CRC_LEN ETHER_CRC_LEN
#endif
#endif

/*
 * XXX hack arp tab cuz I don't want to do the protocol
 */
struct arpentry {
	unsigned char ethaddr[6];
	unsigned char ipaddr[4];
	an_chan_t *chan;
	an_chan_t chan_data;
} arptab[] = {
	/* marker (test) */
	{ { 0x00,0x00,0x92,0xa7,0x54,0x42 }, { 155,99,214,61 }, 0 },
	/* moab (test) */
	{ { 0x00,0x60,0xf5,0x04,0x00,0x53 }, { 155,99,214,88 }, 0 },
	/* moab (prod) */
	{ { 0x00,0xa0,0xc9,0xf1,0x49,0xde }, { 155,99,212,88 }, 0 },
	/* risky */
	{ { 0x00,0x00,0xc0,0x26,0x0c,0x9f }, { 155,99,214,162 }, 0 },
	/* quirky */
	{ { 0x00,0x00,0xc0,0x47,0x0c,0x9f }, { 155,99,214,164 }, 0 },
	/* ibapah */
	{ { 0x00,0xa0,0xc9,0xf1,0x7a,0x06 }, { 155,99,212,83 }, 0 }
};
static int arptabsize = sizeof arptab / sizeof arptab[0];

static void ee_init(void *arg);
an_flowinit_func_t InitEE = { ee_init };

static char *iptoa(unsigned long x);
static void ee_die(an_pbuf_t *pbuf, void *arg);
static an_pbuf_t *diebuf, diebuf_data;
static void ee_arp(an_pbuf_t *pbuf, void *arg);
static an_pbuf_t *arpbuf, arpbuf_data;
static void ee_ip(an_pbuf_t *pbuf, void *arg);
static void setup_channels(void);

static void setup_buffers(int nbufs);
static an_pbuf_t *get_pbuf(void);
static void put_pbuf(an_pbuf_t *pbuf);

static an_demuxkey_t arpdemux_data, *arpdemux, ipdemux_data, *ipdemux;
static an_chan_t *ipinchan, ipinchan_data, *die, die_data;
static an_chan_t *arpinchan, arpinchan_data, *arpoutchan, arpoutchan_data;
static unsigned int myipaddr;
static int dying;

an_ethaddr_t bcastaddr = {
	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff}, {0}, ETHERTYPE_ARP
};
an_ethaddr_t myaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};
an_ethaddr_t arpoutaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_ARP
};
an_ethaddr_t ipoutaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};

int
ee_main(int argc, const char **argv)
{
	assert(argc >= 1);

	printf("%p/%p: %s started\n",
	       an_flow_current(), an_thread_current(), argv[0]);
	myipaddr = *(unsigned int *)myip;

	/*
	 * Setup channels
	 */
	setup_buffers(MAXBUFS);
	setup_channels();

	printf("%s (%02x:%02x:%02x:%02x:%02x:%02x) ready\n",
	       iptoa(*(unsigned int *)myip),
	       myaddr.daddr[0], myaddr.daddr[1], myaddr.daddr[2],
	       myaddr.daddr[3], myaddr.daddr[4], myaddr.daddr[5]);

	an_gprof_start();
#ifdef OSKIT
	{
		extern int memlogging;
		memlogging = 1;
	}
#endif        
	return 0;
}

static void
swapn(unsigned char a[], unsigned char b[], int n)
{
	int i;
	unsigned char t[n];

	for (i = 0; i < n; i++) {
		t[i] = a[i];
		a[i] = b[i];
		b[i] = t[i];
	}
}


/* The address we are passed is in network order. */
static char *
iptoa(unsigned long x)
{
	static char buf[4*4];		/* nnn.nnn.nnn.nnnz */
	unsigned char *p = (unsigned char *)&x;

	sprintf(buf, "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
	return buf;
}

static unsigned short
ipcksum(void *buf, int nbytes)
{
	unsigned long sum;

	sum = 0;
	while (nbytes > 1) {
		sum += *(unsigned short *)buf;
		buf += 2;
		nbytes -= 2;
	}
	/* Add in odd byte if any. */
	if (nbytes == 1)
		sum += *(char *)buf;

	sum = (sum >> 16) + (sum & 0xffff);     /* hi + lo */
	sum += (sum >> 16);                     /* add carry, if any */

	return (~sum);
}

static an_chan_t *
find_channel(struct ip *ip)
{
	static struct arpentry *lastent = arptab;
	struct arpentry *aent;

	/*
	 * Find the table entry
	 */
	aent = lastent;
	do {
		if (memcmp(aent->ipaddr, &ip->ip_src.s_addr, 4) == 0) {
			lastent = aent;
			return aent->chan;
		}
		if (++aent == &arptab[arptabsize])
			aent = arptab;
	} while (aent != lastent);

	return 0;
}

static void
setup_channels(void)
{
	struct arpentry *aent;
	an_error_t err;
	struct ether_arp *arp = 0;
	struct ip *ip = 0;
	unsigned int val_l;
	unsigned short val_s;
	unsigned char val_c;
	struct {
		an_chanaddr_t addr;
		an_chanproto_t proto;
		an_cpuspec_t cpu;
		an_pbuf_t *bufs[MAXBUFS];
	} attr;
	an_flow_t *flow;
	an_ifinfo_t info[8];
	int nif = 8;
	int i;

	flow = an_flow_current();
	memset(&attr, 0, sizeof attr);

	/*
	 * Find my address.
	 * XXX assumes first interface is ethernet.
	 */
	err = an_ifinfo_get(info, &nif);
	assert(err == 0);
	assert(nif != 0);
	assert(info[0].type == ANIF_ETH);
	memcpy(myaddr.daddr, info[0].addr.eth.daddr, sizeof myaddr.daddr);

	/*
	 * Setup a clock channel to time us out in RUNFOR seconds
	 */
	strcpy(attr.proto.data, "clock");
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.clock.freq = (RUNFOR * 100);
	attr.cpu.max = 1;
	attr.bufs[0] = diebuf;
	die = an_inchan_create(&die_data, flow, 0, &attr.proto, &attr.addr,
			       attr.bufs, 1, &attr.cpu, ee_die, 0);
	assert(die != 0);
	printf("%p: clockchannel for %d seconds\n", die, RUNFOR);
	
	/*
	 * Setup in channel for broadcast (arp) packets
	 * First, create a demux to match ARP requests.
	 */
	arpdemux = an_demuxkey_create(&arpdemux_data, an_flow_current());
	assert(arpdemux != 0);
	val_s = htons(ARPHRD_ETHER);
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_hrd, sizeof val_s,
			       &val_s, 0);
	assert(err == 0);
	val_s = htons(ETHERTYPE_IP);
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_pro, sizeof val_s,
			       &val_s, 0);
	assert(err == 0);
	val_s = htons(ARPOP_REQUEST);
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_op, sizeof val_s,
			       &val_s, 0);
	assert(err == 0);
	val_l = myipaddr;
	err = an_demuxkey_match(arpdemux, (int)&arp->arp_tpa, sizeof val_l,
			       &val_l, 0);
	assert(err == 0);

	memset(&attr, 0, sizeof attr);
	sprintf(attr.proto.data, "if%d", info[0].id);
	attr.proto.len = strlen(attr.proto.data);
	bcastaddr.type = htons(bcastaddr.type);
	attr.addr.eth = bcastaddr;
	attr.cpu.max = 1;
	attr.bufs[0] = arpbuf;
	arpinchan = an_inchan_create(&arpinchan_data, flow, arpdemux,
				     &attr.proto, &attr.addr, attr.bufs, 1,
				     &attr.cpu, ee_arp, &info[0]);
	assert(arpinchan != 0);
	printf("%p: ARP inchannel for %s\n", arpinchan, attr.proto.data);

	/*
	 * Setup in channel for ICMP/IP packets
	 * Again, we first build up the demux.  Note that due to the
	 * potentially variable-length IP header, we cannot check the ICMP
	 * type.  So all we can demux on is the IP type and even that might
	 * not be reliable since we cannot verify the checksum.
	 */
	ipdemux = an_demuxkey_create(&ipdemux_data, an_flow_current());
	assert(ipdemux != 0);
	val_c = IPPROTO_ICMP;
	err = an_demuxkey_match(ipdemux, (int)&ip->ip_p, sizeof val_c,
			       &val_c, 0);
	assert(err == 0);

	sprintf(attr.proto.data, "if%d", info[0].id);
	attr.proto.len = strlen(attr.proto.data);
	myaddr.type = htons(myaddr.type);
	attr.addr.eth = myaddr;
	attr.cpu.max = MAXTHREADS;
	for (i = 0; i < MAXTHREADS; i++)
		attr.bufs[i] = get_pbuf();
	ipinchan = an_inchan_create(&ipinchan_data, flow, ipdemux,
				    &attr.proto, &attr.addr,
				    attr.bufs, MAXTHREADS, &attr.cpu, ee_ip, 0);
	assert(ipinchan != 0);

	printf("%p: inchannel for %02x:%02x:%02x:%02x:%02x:%02x, type=%x\n",
	       arpinchan,
	       myaddr.daddr[0], myaddr.daddr[1], myaddr.daddr[2],
	       myaddr.daddr[3], myaddr.daddr[4], myaddr.daddr[5],
	       ntohs(myaddr.type));

	/*
	 * Setup possible output channels
	 */
	ipoutaddr.type = htons(ipoutaddr.type);
	for (aent = arptab; aent < &arptab[arptabsize]; aent++) {
		memset(&attr, 0, sizeof attr);
		sprintf(attr.proto.data, "if%d", info[0].id);
		attr.proto.len = strlen(attr.proto.data);
		memcpy(&ipoutaddr.daddr, aent->ethaddr, sizeof ipoutaddr.daddr);
		attr.addr.eth = ipoutaddr;
		aent->chan = an_outchan_create(&aent->chan_data, flow,
					       &attr.proto, &attr.addr, 0);
		assert(aent->chan != 0);

		printf("%p: outchannel to %s (%02x:%02x:%02x:%02x:%02x:%02x), type=%x\n",
		       aent->chan, iptoa(*(unsigned int *)aent->ipaddr),
		       aent->ethaddr[0], aent->ethaddr[1], aent->ethaddr[2],
		       aent->ethaddr[3], aent->ethaddr[4], aent->ethaddr[5],
		       ntohs(ipoutaddr.type));
	}
}

static void
ee_arp(an_pbuf_t *pbuf, void *arg)
{
	an_ifinfo_t *info = arg;
	int psize = an_pbuf_size(pbuf);
	void *data = an_pbuf_data(pbuf);
	struct ether_arp *arp;
	struct in_addr ip;
	an_error_t err;

	assert(info != 0);

	if (dying)
		goto done;

	if (psize < sizeof *arp) {
#ifdef VERBOSE
		printf("yow! ARP packet too small (%d < %d)\n",
		       psize, sizeof *arp);
#endif
		goto done;
	}


	arp = (struct ether_arp *)data;
	ip.s_addr = *(unsigned long *)arp->arp_tpa;
	assert(ntohs(arp->arp_hrd) == ARPHRD_ETHER);
	assert(ntohs(arp->arp_pro) == ETHERTYPE_IP);
	assert(ntohs(arp->arp_op) == ARPOP_REQUEST);
	assert(ip.s_addr == myipaddr);

	/* Send the reply. */
	arp->arp_op = htons(ARPOP_REPLY);
	swapn(arp->arp_spa, arp->arp_tpa, sizeof arp->arp_tpa);
	swapn(arp->arp_sha, arp->arp_tha, sizeof arp->arp_tha);
	memcpy(arp->arp_sha, myaddr.daddr, sizeof myaddr.daddr);

#ifdef VERBOSE
	printf("sending ARP reply to %s (%02x:%02x:%02x:%02x:%02x:%02x)\n",
	       iptoa(*(unsigned int *)arp->arp_tpa),
	       arp->arp_tha[0], arp->arp_tha[1], arp->arp_tha[2],
	       arp->arp_tha[3], arp->arp_tha[4], arp->arp_tha[5]);
#endif

	/*
	 * Got a legit packet to send.
	 */
	if (memcmp(arpoutaddr.daddr, arp->arp_tha, sizeof arpoutaddr.daddr) != 0) {
		struct {
			an_chanaddr_t addr;
			an_chanproto_t proto;
		} attr;

		if (arpoutchan)
			an_outchan_destroy(arpoutchan);
		else
			arpoutaddr.type = htons(arpoutaddr.type);
		memset(&attr, 0, sizeof attr);
		sprintf(attr.proto.data, "if%d", info->id);
		attr.proto.len = strlen(attr.proto.data);
		memcpy(arpoutaddr.daddr, arp->arp_tha, sizeof arpoutaddr.daddr);
		attr.addr.eth = arpoutaddr;
		arpoutchan = an_outchan_create(&arpoutchan_data,
					       an_flow_current(),
					       &attr.proto, &attr.addr, 0);
		assert(arpoutchan != 0);
	}
	err = an_outchan_send(arpoutchan, pbuf);
	assert(err == 0);

done:
	an_inchan_addpbuf(arpinchan, pbuf);
}

static void
ee_ip(an_pbuf_t *pbuf, void *arg)
{
	int psize = an_pbuf_size(pbuf);
	void *data = an_pbuf_data(pbuf);
	int ipdata[ETHER_MAX_LEN/sizeof(int)+1];
	static int ipid = 0xb;		/* something random */
	struct ip *ip;
	int hlen;
	struct icmp *icmp;
	an_chan_t *outchan;
	an_error_t err;

	if (dying)
		goto done;

	an_inchan_addpbuf(ipinchan, get_pbuf());

	if (psize < sizeof *ip) {
#ifdef VERBOSE
		printf("yow! IP packet too small (%d < %d)\n",
		       psize, sizeof *ip);
#endif
		goto done;
	}

	/* XXX deal with fragments. */

	ip = (struct ip *)data;
	hlen = ip->ip_hl << 2;

	/*
	 * Copy out from ip header on to ensure proper alignment
	 * on machines that are not byte addressable!
	 */
	memcpy(ipdata, data, ntohs(ip->ip_len));
	ip = (struct ip *) ipdata;

	assert(ip->ip_p == IPPROTO_ICMP);
	if (ipcksum(ip, hlen) != 0)
		goto done;		/* bad cksum */

	icmp = (struct icmp *)((char *)ip + hlen);
	if (icmp->icmp_type != ICMP_ECHO
	    || ipcksum(icmp, ntohs(ip->ip_len) - hlen) != 0)
		goto done;		/* bad cksum or not ICMP_ECHO */

	/*
	 * Find/create a channel to send on
	 */
	outchan = find_channel(ip);
	if (outchan == 0) {
#ifdef VERBOSE
		printf("yow! I do not know about %s\n",
		       iptoa(ip->ip_src.s_addr));
#endif
		goto done;
	}

	/* Send the reply. */
	icmp->icmp_type = ICMP_ECHOREPLY;
	/* Another optimization.  Only the icmp type field in the reply
	 * changes, so we can use this apriori knowledge to determine
	 * the new checksum based upon the old checksum.  Eliminates
	 * touching all of the data in the packet once more */
#if 0
	icmp->icmp_cksum = 0;
	icmp->icmp_cksum = ipcksum(icmp, ntohs(ip->ip_len) - hlen);
#else
	if (icmp->icmp_cksum >= 65527)
		icmp->icmp_cksum += (unsigned short) 9;
	else
		icmp->icmp_cksum += (unsigned short) 8;
#endif

	ip->ip_id = htons(ipid++);

	/*
	 * We can't just swap the addresses,
	 * since broadcast packets didn't come to our address.
	 */
	ip->ip_dst = ip->ip_src;
	ip->ip_src.s_addr = myipaddr;

	ip->ip_sum = 0;
	ip->ip_sum = ipcksum(ip, hlen);

	/*
	 * Copy data back into original data buffer.
	 */
	memcpy(data, ip, ntohs(ip->ip_len));

#ifdef VERBOSE
	printf("sending ICMP reply to %s on %p\n",
	       iptoa(ip->ip_dst.s_addr), outchan);
#endif

	err = an_outchan_send(outchan, pbuf);
	assert(err == 0);

done:
	put_pbuf(pbuf);
}

static void
ee_die(an_pbuf_t *pbuf, void *arg)
{
	an_chanstats_t instats, outstats;
	an_error_t err;

	dying = 1;
	an_gprof_stop();
	err = an_chan_stats(ipinchan, &instats);
	assert(err == 0);
	err = an_chan_stats(ipinchan, &outstats);
	assert(err == 0);
	printf("All done!\n");
	printf("Request: %d pkts (%d pkts/sec), %d bytes (%d bytes/sec)\n",
	       instats.inpackets, instats.inpackets/RUNFOR,
	       instats.inbytes, instats.inbytes/RUNFOR);
	printf("Reply: %d pkts (%d pkts/sec), %d bytes (%d bytes/sec)\n",
	       outstats.inpackets, outstats.inpackets/RUNFOR,
	       outstats.inbytes, outstats.inbytes/RUNFOR);
	printf("Drops: %d pkts\n", instats.droppackets);
	if (ipinchan)
		an_inchan_destroy(ipinchan);
	if (arpoutchan)
		an_outchan_destroy(arpoutchan);
	if (arpinchan)
		an_inchan_destroy(arpinchan);
	an_flow_destroy(an_flow_current());
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "PingreplyEE" }; 

	status = ee_main(argc, argv);
}

/*
 * Packet buffer management
 */

struct buflist {
	an_pbuf_t buf;	/* must be first */
	struct buflist *next;
};
static struct buflist *freelist, *buffers;
static an_mutex_t *buflock, buflock_data;

void
setup_buffers(int nbufs)
{
	an_pbuf_t *bufptr;
	size_t size;
	int i;

	diebuf = an_pbuf_create(&diebuf_data, AN_PBUF_CLOCK, 0, 0);
	assert(diebuf != 0);
	arpbuf = an_pbuf_create(&arpbuf_data, AN_PBUF_ETH, 0, 0);
	assert(arpbuf != 0);

	buflock = an_mutex_create(&buflock_data);
	an_mutex_lock(buflock);

	size = nbufs * sizeof(struct buflist);
	buffers = an_mempool_alloc(an_mempool_current(), size);
	assert(buffers != 0);
	for (i = 0; i < nbufs; i++) {
		bufptr = an_pbuf_create(&buffers[i].buf, AN_PBUF_ETH, 0, 0);
		assert(bufptr == &buffers[i].buf);
		buffers[i].next = freelist;
		freelist = &buffers[i];
	}

	an_mutex_unlock(buflock);
}

an_pbuf_t *
get_pbuf()
{
	an_pbuf_t *pbuf;

	an_mutex_lock(buflock);
	if (freelist != 0) {
		pbuf = &freelist->buf;
		freelist = freelist->next;
	} else
		pbuf = 0;
	an_mutex_unlock(buflock);

	return pbuf;
}

void
put_pbuf(an_pbuf_t *pbuf)
{
	struct buflist *bl = (struct buflist *)pbuf;

	an_mutex_lock(buflock);
	bl->next = freelist;
	freelist = bl;
	an_mutex_unlock(buflock);
}
