/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Event API
 */
#ifndef _NODEOS_EVENT_H_
#define _NODEOS_EVENT_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Event an_event_t
#define an_EventState an_eventstate_t
#define an_eventSchedule an_event_schedule
#define an_eventDetach an_event_detach
#define an_eventCancel an_event_cancel
#define an_eventState an_event_state
#define an_eventIsCanceled an_event_canceled
#endif

#include <sys/types.h>
#include <nodeos/an_flow.h>

typedef struct {
	int stuff;
} *an_event_t;

typedef enum {
	AN_EVENT_INVALID = -1,
	AN_EVENT_PENDING,
	AN_EVENT_RUNNING,
	AN_EVENT_FINISHED,
	AN_EVENT_CANCELED
} an_eventstate_t;

typedef void (*an_eventfunc_t)(an_event_t, void *);

/*
 * Schedule an event to occur at least the indicated number of microseconds
 * in the future.  The given handler is run in the context of the specified
 * flow and is provide with the given argument.  If an event pointer is
 * provided, this call will return a handle with which other event operations
 * can be performed.  If no pointer is provided, the event is "detached",
 * running as scheduled and then automatically cleaning up.
 *
 * ``event'' if non-zero, filled in with a handle for the event,
 * ``flow'' is the flow context in which to run the handler,
 * ``func'' is the function to invoke when the specified time has elapsed,
 * ``farg'' is the argument for the handler function,
 * ``delta'' is the time (in usec) in the future at which to run the handler.
 *
 * Returns the new event or zero on an error.
 */
an_error_t an_event_schedule(an_event_t *event, an_flow_t flow,
			     an_eventfunc_t func, void *farg,
			     unsigned long delta);

/*
 * Cancel the indicated event if it has not yet occured.  Canceling an
 * event implicitly detaches it as well; i.e., no further operations should
 * be performed on it and its resources will be reclaimed.  The effect of
 * canceling a detached event is undefined.  Canceling running, finished or already canceled events
 * has no effect.  The effect of canceling a detached event is undefined.
 * Returns the status of the event after the cancellation operation:
 * AN_EVENT_CANCELED, AN_EVENT_RUNNING, AN_EVENT_
 */

#endif /* _NODEOS_EVENT_H_ */
