/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation bits of NodeOS Condition Variable API.
 */
#ifndef _NODEOS_IMPL_COND_H_
#define _NODEOS_IMPL_COND_H_

#include <nodeos/impl/decls.h>
#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_cond.h>

/*
 * an_cond interfaces.
 *
 * an_cond vars are a simple wrapper around the ani_cond layer.
 * This layer add the NODEOS entry/exit and checking to the 
 * ani_cond.
 *
 * The Utah NodeOS inlines the signal(), broadcast(), wait() and timedwait()
 * operations.  The init() and destroy() operations are not inlined.
 */

typedef struct an_cond {
	ani_cond_t cond;	/* must be first for INTERN/EXTERN */
} an_cond_t;

/* cond inline functions */

NODEOS_FUNC_INLINE ani_cond_t *
NODEOS_INTERN_COND(an_cond_t *c)
{
	assert(c != 0);
	return &c->cond;
}

NODEOS_FUNC_INLINE an_cond_t *
NODEOS_EXTERN_COND(ani_cond_t *ic)
{
	assert(ic != 0);
	return (an_cond_t *)ic;
}

NODEOS_FUNC_INLINE void 
an_cond_wait(an_cond_t *c, an_mutex_t *m)
{
	NODEOS_ENTER;
	ani_cond_wait(&c->cond, NODEOS_INTERN_MUTEX(m));
	NODEOS_LEAVE;
}

NODEOS_FUNC_INLINE void
an_cond_timedwait(an_cond_t *c, an_mutex_t *m, an_timespec_t *abstime)
{
	NODEOS_ENTER;
	ani_cond_timedwait(&c->cond, NODEOS_INTERN_MUTEX(m), abstime);
	NODEOS_LEAVE;
}

NODEOS_FUNC_INLINE void
an_cond_signal(an_cond_t *c)
{
	NODEOS_ENTER;
	ani_cond_signal(&c->cond);
	NODEOS_LEAVE;
}

NODEOS_FUNC_INLINE void
an_cond_broadcast(an_cond_t *c)
{
	NODEOS_ENTER;
	ani_cond_broadcast(&c->cond);
	NODEOS_LEAVE;
}

#endif /* _NODEOS_IMPL_COND_H_ */
