/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _DEMUXKEY_IMPL_H_
#define _DEMUXKEY_IMPL_H_

/*
 * A list of <off, len, key> triples.  A packet matches if each of the
 * triples match.  A triple matches if "len" bytes of the packet data
 * at offset "off" are equal to "key".
 */
#define ANI_DEMUXKEY_MAX 6

struct demux_triple;

struct ani_demuxkey {
	int maxoff;	/* highest data offset covered by the list */
	short ntriples;	/* number of tuples in the list used */
	short listsize;	/* size of list */
	struct demux_triple *list[ANI_DEMUXKEY_MAX];
};
typedef struct ani_demuxkey ani_demuxkey_t;

int ani_demuxkey_init(ani_demuxkey_t *list);
int ani_demuxkey_deinit(ani_demuxkey_t *list);
int ani_demuxkey_clone(ani_demuxkey_t *nlist, ani_demuxkey_t *olist);
int ani_demuxkey_prepend(ani_demuxkey_t *list, int off, int len, void *key);
int ani_demuxkey_append(ani_demuxkey_t *list, int off, int len, void *key);
int ani_demuxkey_expand(ani_demuxkey_t *list, int nelts);
void ani_demuxkey_adjust(ani_demuxkey_t *list, int adjust);

int ani_demuxkey_match(void *ddata, void *pdata, int plen);
void ani_demuxkey_dump(ani_demuxkey_t *list);

#endif /* _DEMUXKEY_IMPL_H_ */
