/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _HNODE_H_
#define _HNODE_H_

/*
 * A ``hierarchy'' node.
 * Each node has a link to its parent, its next sibling and its first child.
 * Each node can also be on another ``flat'' list, with a pointer to the head
 * of the list and to the next element in the list.
 */

struct hnode {
	struct hnode *parent;		/* our parent */
	struct hnode *parentlink;	/* our next sibling */
	struct hnode *child;		/* our first child */
	struct hnode *global;		/* global list we are on */
	struct hnode *globallink;	/* link to siblings on global list */
};

void hnode_init(struct hnode *node);
void hnode_attach(struct hnode *node, struct hnode *parent, struct hnode *list);
void hnode_detach(struct hnode *node, int flags);
int hnode_traverse(struct hnode *root, int (*func)(struct hnode *, void *),
		   void *arg, int flags, int reverse);
struct hnode *hnode_parent(struct hnode *node);
struct hnode *hnode_child(struct hnode *node);
struct hnode *hnode_sibling(struct hnode *node);
struct hnode *hnode_listhead(struct hnode *node);
struct hnode *hnode_listnext(struct hnode *node);

/* hnode_remove flags */
#define HNODE_NONE	0
#define HNODE_PARENT	1
#define HNODE_GLOBAL	2
#define HNODE_ALL	(~0)

#endif /* _HNODE_H_ */
