/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#ifndef _NODEOS_IMPL_ANI_MUTEX_H_
#define _NODEOS_IMPL_ANI_MUTEX_H_

/*
 * ani_mutex_*
 *
 * ani_mutexes are currently trivial wrappers around the ani_ilock.
 * Eventually this layer may add cancelability to the ani_mutex.
 *
 * The lock(), unlock() and trylock() functions are inlined.  init()
 * and deinit() are in src/ani_mutex.c.
 */

#include <nodeos/impl/ani_ilock.h>
#include <nodeos/impl/decls.h>

typedef struct ani_mutex_t {
	ani_ilock_t ilock;
} ani_mutex_t;

/* prototypes */

int ani_mutex_init(ani_mutex_t *mux);

int ani_mutex_deinit(ani_mutex_t *mux);


/* inline functions */

NODEOS_FUNC_INLINE void
ani_mutex_lock(ani_mutex_t *mux)
{
	ani_ilock_lock(&(mux->ilock));
}

NODEOS_FUNC_INLINE void
ani_mutex_unlock(ani_mutex_t *mux)
{
	ani_ilock_unlock(&(mux->ilock));
}

NODEOS_FUNC_INLINE int
ani_mutex_trylock(ani_mutex_t *mux)
{
	return ani_ilock_trylock(&(mux->ilock));
}

#endif /* _NODEOS_IMPL_ANI_MUTEX_H_ */
