/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Macros and inline functions for the NodeOS system call
 * entry/exit layer and for the intern/extern of various types.
 */
#ifndef _NODEOS_IMPL_NODEOS_H_
#define _NODEOS_IMPL_NODEOS_H_

#include <string.h>
#include <assert.h>
#include <signal.h>

#include <nodeos/impl/ani_thread.h>

/*
 * We disallow both thread interrupts (signals) and thread death (cancelation)
 * while in the NodeOS.  Both are checked on exit from the NodeOS.
 */
#define NODEOS_ENTER { \
	ani_thread_t *th = ani_thread_current(); \
	int ostate; \
	extern sigset_t ani_thread_intrmask; \
	pthread_sigmask(SIG_BLOCK, &ani_thread_intrmask, 0); \
	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &ostate); \
	assert(ostate == PTHREAD_CANCEL_ASYNCHRONOUS); \
	assert((th->flags & THREAD_INKERNEL) == 0); \
	th->flags |= THREAD_INKERNEL; \
}

#define NODEOS_LEAVE { \
	ani_thread_t *th = ani_thread_current(); \
	int ostate; \
	extern sigset_t ani_thread_intrmask; \
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &ostate); \
	assert(ostate == PTHREAD_CANCEL_DEFERRED); \
	pthread_testcancel(); \
	assert((th->flags & THREAD_INKERNEL) != 0); \
	th->flags &= ~THREAD_INKERNEL; \
	if (th->flags & THREAD_INTSON) \
		pthread_sigmask(SIG_UNBLOCK, &ani_thread_intrmask, 0); \
}

/*
 * For many API calls, it doesn't matter if a thread is interrupted or
 * cancelled as there are no dangerous side-effects (e.g., taking a lock)
 * and the calls cannot block.  For these calls, we don't even bother to
 * record that we are in the NodeOS.
 */
#define LW_NODEOS_ENTER
#define LW_NODEOS_LEAVE

#define NODEOS_COPYIN(ndst, usrc, len, errp) { \
	memcpy((ndst), (usrc), (len)); \
	*errp = 0; \
}

#define NODEOS_COPYOUT(udst, nsrc, len, errp) { \
	memcpy((udst), (nsrc), (len)); \
	*errp = 0; \
}

#endif /* _NODEOS_IMPL_NODEOS_H_ */
