/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS cond entrypoints.
 */

#include <nodeos/an_cond.h>
#include <nodeos/an_mutex.h>
#include <nodeos/an_error.h>
#include <nodeos/impl/ani_mutex.h>
#include <nodeos/impl/ani_cond.h>
#include <nodeos/impl/ani_flow.h>
#include <nodeos/impl/ani_mempool.h>

/*
 * Initalize an existing piece of memory to be a cond object.
 */
an_cond_t *
an_cond_create(an_cond_t *mem)
{
	ani_cond_t *cond;
	ani_mempool_t *cpool;
	int err, didalloc = 0;

	NODEOS_ENTER;

	/*
	 * Allocate memory for the object if needed.
	 */
	cpool = ani_flow_mempool(ani_flow_current());
	if (mem == 0) {
		mem = ani_mempool_alloc(cpool, sizeof(an_cond_t));
		didalloc = 1;
	}

	cond = NODEOS_INTERN_COND(mem);
	if (cond == 0)
		err = AN_ERR_NOMEMORY;
	else
		err = ani_cond_init(cond);

	if (err != 0 && didalloc)
		ani_mempool_free(cpool, mem, sizeof(an_cond_t));

	NODEOS_LEAVE;
	return err ? 0 : NODEOS_EXTERN_COND(cond);
}

/* an_cond_signal is inlined from <nodeos/impl/synch.h> */
/* an_cond_broadcast is inlined from <nodeos/impl/synch.h> */
/* an_cond_wait is inlined from <nodeos/impl/synch.h> */
/* an_cond_timedwait is inlined from <nodeos/impl/synch.h> */

an_error_t
an_cond_destroy(an_cond_t *cond)
{
	int err;
	ani_cond_t *icond;

	NODEOS_ENTER;
	icond = NODEOS_INTERN_COND(cond);
	if (icond == 0)
		err = AN_ERR_INVALIDARG;
	else
		err = ani_cond_deinit(icond);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}
