/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Packet API
 */
#include <nodeos/an_error.h>
#include <nodeos/an_demuxkey.h>
#include <nodeos/impl/ani_nodeos.h>

#include <nodeos/impl/ani_flow.h>
#include <nodeos/impl/ani_mempool.h>
#include <nodeos/impl/ani_demuxkey.h>

an_demuxkey_t *
an_demuxkey_create(void *mem, an_flow_t *flow)
{
	ani_demuxkey_t *ikey;
	ani_flow_t *iflow;
	int err;

	NODEOS_ENTER;
	ikey = NODEOS_INTERN_DEMUXKEY(mem);
	iflow = NODEOS_INTERN_FLOW(flow);
	if (ikey == 0 || iflow == 0)
		err = AN_ERR_INVALIDARG;
	else {
		assert(iflow == ani_flow_current());
		err = ani_demuxkey_init(ikey);
	}
	NODEOS_LEAVE;
	return err ? 0 : NODEOS_EXTERN_DEMUXKEY(ikey);
}

an_error_t
an_demuxkey_destroy(an_demuxkey_t *key)
{
	int err;

	NODEOS_ENTER;
	err = ani_demuxkey_deinit(NODEOS_INTERN_DEMUXKEY(key));
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

an_demuxkey_t *
an_demuxkey_clone(void *mem, an_demuxkey_t *orig)
{
	ani_demuxkey_t *inew, *iorig;
	int err;

	NODEOS_ENTER;
	inew = NODEOS_INTERN_DEMUXKEY(mem);
	iorig = NODEOS_INTERN_DEMUXKEY(orig);
	if (inew == 0 || iorig == 0)
		err = AN_ERR_INVALIDARG;
	else
		err = ani_demuxkey_clone(inew, iorig);
	NODEOS_LEAVE;
	return err ? 0 : NODEOS_EXTERN_DEMUXKEY(inew);
}

an_error_t
an_demuxkey_match(an_demuxkey_t *key, int off, int len, void *datap,
		  void *maskp)
{
	ani_demuxkey_t *ikey;
	ani_mempool_t *mpool;
	void *data = 0;
#if 0
	void *mask = 0;
#endif
	int err = 0;

	NODEOS_ENTER;

	mpool = ani_flow_mempool(ani_flow_current());
	assert(mpool != 0);

	ikey = NODEOS_INTERN_DEMUXKEY(key);
	if (ikey == 0) {
		err = AN_ERR_INVALIDARG;
		goto done;
	}

	if (datap) {
		data = ani_mempool_alloc(mpool, len);
		if (data) {
			NODEOS_COPYIN(data, datap, len, &err);
		} else
			err = AN_ERR_NOMEMORY;
	} else
		err = AN_ERR_INVALIDARG;
	if (err)
		goto done;

#if 0
	if (maskp) {
		mask = ani_mempool_alloc(mpool, len);
		if (mask) {
			NODEOS_COPYIN(mask, maskp, len, &err);
		} else
			err = AN_ERR_NOMEMORY;
	} else
		mask = 0;
	if (err)
		goto done;
#endif

	err = ani_demuxkey_append(ikey, off, len, data);

done:
	if (err != 0) {
		if (data != 0)
			ani_mempool_free(mpool, data, len);
#if 0
		if (mask != 0)
			ani_mempool_free(mpool, mask, len);
#endif
	}
	NODEOS_LEAVE;

	return NODEOS_EXTERN_ERROR(err);
}
