/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Mempool API
 */
#include <nodeos/an_error.h>
#include <nodeos/an_impl.h>
#include <nodeos/an_mempool.h>

#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_mempool.h>

an_mempool_t *
an_mempool_create(void *mem, an_mempool_t *parent, an_mempoolfull_func_t func)
{
	ani_mempool_t *newpool;
	ani_mempool_t *iparent = NULL;
	int err;

	NODEOS_ENTER;

	newpool = NODEOS_INTERN_MEMPOOL(mem);
	if (newpool == 0) {
		err = AN_ERR_INVALIDARG;
		goto done;
	}

	if (parent) {
		iparent = NODEOS_INTERN_MEMPOOL(parent);
		if (iparent == 0) {
			err = AN_ERR_INVALIDARG;
			goto done;
		}
	}

	err = ani_mempool_init(newpool, iparent, func);

 done:
	NODEOS_LEAVE;
	return err ? 0 : NODEOS_EXTERN_MEMPOOL(newpool);
}

an_error_t
an_mempool_destroy(an_mempool_t *mpool)
{
	ani_mempool_t *impool;
	int err = 0;

	NODEOS_ENTER;
	impool = NODEOS_INTERN_MEMPOOL(mpool);
	if (impool == 0) {
		err = AN_ERR_INVALIDARG;
		goto done;
	}
	err = ani_mempool_deinit(impool);
 done:
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

void *
an_mempool_alloc(an_mempool_t *mpool, size_t size)
{
	ani_mempool_t *impool;
	void *ptr = 0;

	NODEOS_ENTER;
	impool = NODEOS_INTERN_MEMPOOL(mpool);
	if (impool != 0)
		ptr = ani_mempool_alloc(impool, size);
	NODEOS_LEAVE;
	return ptr;
}

void
an_mempool_free(an_mempool_t *mpool, void *ptr, size_t size)
{
	ani_mempool_t *impool;

	NODEOS_ENTER;
	impool = NODEOS_INTERN_MEMPOOL(mpool);
	if (impool != 0)
		ani_mempool_free(impool, ptr, size);
	NODEOS_LEAVE;
}
