/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include <stdio.h>
#include <stdarg.h>
#include <signal.h>
#include <pthread.h>
#include <assert.h>
#include <sys/time.h>

#include "os.h"
#include "options.h"

#ifdef CHAN_ETH
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>

#include <net/if.h>
#include <net/bpf.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#include <sys/uio.h>

#include <nodeos/impl/an_error.h>
#include <nodeos/impl/ani_nodeos.h>
#endif /* CHAN_ETH */

void
os_init_basic(void)
{
}

void
os_init_mem(void)
{
}

void
os_init_devices(void)
{
}

/*
 * Set the priority of the given thread.
 * This routine should map standard NodeOS priority values into
 * implementation specific priorities.
 */
int
os_sched_setprio(ani_thread_t *thread, int nprio)
{
	return 0;
}

/*
 * This function is called by the assert() macro defined in assert.h;
 * it's also a nice simple general-purpose panic function.
 */
void
panic(const char *fmt, ...)
{
	va_list vl;

	va_start(vl, fmt);
	vprintf(fmt, vl);
	va_end(vl);

	printf("\n");

	exit(1);
}

void
os_dump_state(void *state)
{
}

#ifdef CHAN_ETH

static int
bpf_open(void)
{
	int fd;
	int n = 0;
	char dev[sizeof("/dev/bpf000")];
	
	do {
		(void) sprintf(dev, "/dev/bpf%d", n++);
		fd = open(dev, O_RDWR);
	} while (fd < 0 && errno == EBUSY);

	return fd;
}

int
os_open_eth(char *dev)
{
	int fd;
	int buflen;
	struct ifreq ifr;

	if ( (fd = bpf_open()) < 0)
		return -1;

	buflen = NODEOS_CHAN_ETH_BSIZE;

	if (ioctl(fd, BIOCSBLEN, &buflen) < 0) {
		close(fd);	
		return -1;
	}

	/*
	 * Bind our bpf device to the interface.
	 */
	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
	if (ioctl(fd, BIOCSETIF, &ifr) < 0) {
		close(fd);
		return -1;
	}

#if 0
	/*
	 * If we ever need promiscuous mode, this is how to turn it on. 
	 */
	if (ioctl(fd, BIOCPROMISC, &ifr) < 0) {
		close(fd);
		return -1;
	}
#endif	    

	return fd;
}

void
os_close_eth(int fd)
{
	close(fd);
}

int
os_read_packet(int fd, char *buf, int bufsize)
{
	struct bpf_hdr *hdr;
	static char ourbuf[NODEOS_CHAN_ETH_BSIZE];
	static int ofs = 0;
	static int count = 0;
	int len = 0;

	if (ofs == 0) {
		while ( (count = read(fd, &ourbuf[0], bufsize)) < 0) {
			if (errno == EWOULDBLOCK) 
				return 0;
			if (errno != EINTR)
				return -1;
		}
	}
		
	if (count != 0) {
		hdr = (struct bpf_hdr *)(&ourbuf[0] + BPF_WORDALIGN(ofs));
		len = hdr->bh_caplen;

		if (hdr->bh_caplen != hdr->bh_datalen)
			printf("%s: truncated packet!\n", __FUNCTION__);
		
		memcpy(buf, &ourbuf[0] + ofs + hdr->bh_hdrlen, len);

		ofs += hdr->bh_hdrlen + hdr->bh_caplen;
		ofs = BPF_WORDALIGN(ofs);

		if (ofs >= count) 
			ofs = count = 0;
	}

	return len;
}

int
os_write_packet(int fd, char *buf, int bufsize, struct ether_header *eh)
{
	ssize_t count;
	struct iovec msg_iov[2];
	int len;

	if (eh != 0) {
		msg_iov[0].iov_base = (char *)eh;
		msg_iov[0].iov_len  = ETHER_HDR_LEN;
		msg_iov[1].iov_base = buf;
		msg_iov[1].iov_len  = bufsize;
		len = 2;
	} else {
		msg_iov[0].iov_base = buf;
		msg_iov[0].iov_len  = bufsize;
		len = 1;
	}
	
	count = writev(fd, (struct iovec *)&msg_iov, len);
	if (count < 0)
		perror("problem writing packet");
	return count;
}
#endif /* CHAN_ETH */
