/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include <nodeos/an_resource.h>
#include <nodeos/an_flow.h>

#include <nodeos/impl/ani_thread.h>
#include <nodeos/impl/ani_ilock.h>
#include <nodeos/impl/ani_cred.h>
#include <nodeos/impl/ani_flow.h>
#include <nodeos/impl/ani_mempool.h>
#include <nodeos/impl/ani_chan.h>
#include "mem.h"
#include "os.h"

static void load_ee(void *);
static void nodeos_die(an_flow_t *);

#include "version.h"
int version = VERSION;

static ani_flow_t flow;
static ani_icond_t cond;
static ani_ilock_t lock;
static int childdead;
int bootstrapped;

/*
 * XXX hide these two behind some impl.h interface.
 */
int main_argc;
char **main_argv;

int
main(int argc, char **argv)
{
	os_init_basic();
#ifdef GPROF
	os_gprof_init();
#endif
	/*
	 * XXX Save off argc and argv where the EE can find them.
	 */
	main_argc = argc;
	main_argv = argv;

	os_init_mem();
	printf("Utah NodeOS (version 0.%04d)\n", version);
	os_init_devices();
	os_init_fs();

	/*
	 * Initialize NodeOS components.
	 */
	mem_init();
	ani_thread_setup();
	ani_cred_init();
	ani_mempool_setup();
	ani_chan_setup();

	ani_ilock_init(&lock);
	ani_icond_init(&cond);
	ani_ilock_lock(&lock);

	/*
	 * Create our initial flow.
	 * This will fire off a thread at load_ee which takes over from here.
	 * All memory from this point on is charged to some flow.
	 */
	ani_flow_setup(load_ee, nodeos_die);

	/*
	 * Wait til the child flow dies
	 */
	while (childdead == 0)
		ani_icond_wait(&cond, &lock);
	bootstrapped = 0;
	printf("Initial EE died, exiting...\n");

	/*
	 * XXX wait a while longer to allow things to shutdown cleanly.
	 */
	{
		an_timespec_t ts;

		ts.tv_sec = 1;
		ts.tv_nsec = 0;
		ani_thread_sleep(&ts);
	}

	exit(0);
}

static void
load_ee(void *arg)
{
	void (*func)(void *);
	ani_cred_t *cred;
	an_cpuspec_t cspec;
	an_memspec_t mspec;
	int err;

	bootstrapped = 1;
	printf("Loading initial EE...\n");
#if 1
	{
		extern ani_cred_t *user_cred;
		extern an_flowinit_func_t InitEE;

		func = InitEE;
		cred = user_cred;
		memset(&cspec, 0, sizeof cspec);
		cspec.max = 100;
		cspec.stacksize = 32 * 1024;
		mspec.npages = 1000;
		mspec.pool = NODEOS_EXTERN_MEMPOOL(ani_mempool_current());
	}
#endif
	err = ani_flow_init(&flow, cred, &cspec, &mspec, func, 0, nodeos_die);
	assert(err == 0);

	ani_thread_exit();
}

static void
nodeos_die(an_flow_t *flow)
{
	ani_ilock_lock(&lock);
	childdead = 1;
	ani_icond_signal(&cond);
	ani_ilock_unlock(&lock);
}
