/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Miscellaneous functions
 */
#include <sys/time.h>
#include <assert.h>

#include "misc.h"

/*
 * Return the currert time in POSIX format.
 * XXX gettimeofday is not POSIX but beats the hell out of "time"
 */
void
time_current(an_timespec_t *curtime)
{
	struct timeval tv;
	int err;

	err = gettimeofday(&tv, 0);
	assert(err == 0);
	curtime->tv_sec = tv.tv_sec;
	curtime->tv_nsec = tv.tv_usec * 1000;
}

/*
 * Return a timespec reflecting delta sec/nsec in the future.
 * Used to create values for pthread_cond_timedwait which wants
 * an absolute timeout value.
 */
void
time_future(an_timespec_t *result, an_timespec_t *delta)
{
	time_current(result);
	result->tv_sec += delta->tv_sec;
	result->tv_nsec += delta->tv_nsec;
	if (result->tv_nsec >= 1000000000) {
		result->tv_nsec -= 1000000000;
		result->tv_sec++;
	}
}
