#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

TARGET =	libnodeos.a
TARGET_P =	libnodeos_p.a

SRCS =		main.c flow.c threadpool.c thread.c credential.c resource.c \
		mempool.c mem.c channel.c pbuffer.c synch.c demux.c \
		demuxkey.c hnode.c misc.c if.c \
		chan_clock.c chan_eth.c chan_socket.c \
		an_mutex.c an_cond.c \
		an_cred.c an_flow.c an_thread.c an_mempool.c \
		an_chan.c an_demuxkey.c an_pbuf.c \
		an_misc.c an_gprof.c \
		uninlined.c

include $(NODEOS_SRCDIR)/arch.mk

ifeq ($(HOST_OS),oskit)
ARCH_SRCS = 	oskit.c oskit_mem.c oskit_net.c \
		oskit_gprof.c oskit_fs.c
endif

ifeq ($(HOST_OS),oskitunix)
ARCH_SRCS =	oskitunix.c oskitunix_mem.c oskitunix_dev.c \
		oskitunix_gprof.c oskitunix_fs.c
endif

ifeq ($(HOST_OS),linux-gnu)
ARCH_SRCS =	linux.c unix_eth.c unix_gprof.c unix_fs.c
endif

ifeq ($(HOST_OS),solaris2.7)
ARCH_SRCS =	solaris.c unix_eth.c unix_gprof.c unix_fs.c
endif

ifeq ($(HOST_OS),freebsd3.0)
ARCH_SRCS =	freebsd.c unix_eth.c unix_gprof.c unix_fs.c
endif

FILES = 	$(SRCS) $(ARCH_SRCS)
FQSRCS =	$(addprefix $(SRCDIR)/,$(sort $(FILES)))
DEPFILES:=	$(patsubst %.c,%.d,$(FILES))

OBJS := $(patsubst %.c,%.o,$(notdir $(FQSRCS)))
POBJS := $(patsubst %.c,%.po,$(notdir $(FQSRCS)))

#CFLAGS_WARNINGS=-W -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wnested-externs -Winline
CFLAGS_WARNINGS=-Wcast-align -Wwrite-strings -Wstrict-prototypes -Winline

CPPFLAGS = $(OPT_CFLAGS) $(ARCH_CFLAGS) -I$(NODEOS_SRCDIR)
CFLAGS += -Wall $(CFLAGS_WARNINGS) $(CPPFLAGS)

ifeq ($(ENABLE_PROFILING),yes)
all: all_ all_p
install: install_ install_p
else
all: all_
install: install_
endif

all_: $(TARGET)

all_p: $(TARGET) $(TARGET_P)

install_dirs:
	@test -d $(INSTALL_TOPDIR) || mkdir $(INSTALL_TOPDIR)
	@test -d $(INSTALL_LIBDIR) || mkdir $(INSTALL_LIBDIR)

install_: install_dirs $(TARGET)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET)."
	$(INSTALL_DATA) $(TARGET) $(INSTALL_LIBDIR)

install_p: install_dirs $(TARGET_P)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET_P)."
	$(INSTALL_DATA) $(TARGET_P) $(INSTALL_LIBDIR)


$(TARGET): $(OBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

$(TARGET_P): $(POBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

clean:
	$(SILENT_MAKE_INFORM) "Removing *.o *.po $(TARGET) $(TARGET_P)."
	rm -f .depend core *.core *.d *.o *.po $(TARGET) $(TARGET_P)

distclean: clean
	$(SILENT_MAKE_INFORM) "Scrubbing nodeos library directory."
	rm -f GNUmakefile

depend: 
	@echo "WARNING: depend target is deprectated."

-include $(DEPFILES)

.PHONY: depend distclean clean 
.PHONY: all all_ all_p
.PHONY: install install_dirs install_ install_p 

